/**
* HTMLBuilder
*
* A micro-toolkit designed to make creating/manipulating DOM Elements
* fast and easy.
*
* This file contains the builder component.
*
* Copyright 2010, Carlos Killpack
* Dual licensed under New BSD and WTFPL licenses
*/
if (!(typeof Array.prototype.forEach === 'function')) {
    Array.prototype.forEach = function (fn) {
        var self = this;
        for (var i = 0, l = self.length; i < l; i += 1) {
            fn(self[i]);
        }
    };
}
if (!(typeof Object.keys === 'function')) {
    Object.keys = function (o) {
        var key, keys = [];
        for (key in o) {
            o.hasOwnProperty(key) && keys.push(key);
        }
        return keys;
    };
}
if (!(typeof Array.isArray === 'function')) {
    Array.isArray = function (o) {
        return Object.prototype.toString.call(o) === '[object Array]';
    };
}
/**
 * @constructor
 * A component of HTMLBuilder that builds, inserts, and destorys DOM
 * elements.
 */
window['builder'] = new function Builder () {
    "use strict";
    var self = this;
    self.version = '0.6';
    /**
     * Creates an element, sets its properties, and inserts
     * its contents. Takes the following array literal
     * as an argument, optionally wrapped in an array, which allows you
     * create multiple sibling elements:
     * 
     * [
     *     <tagname>,
     *     {<key>: <value>},
     *     <content> || [...],
     *     <content> || [...],
     *     ...
     * ]
     *
     * Everything is optional with the exception of <tagname>.
     */
    self.build = function (elDef, callback) {
        if (Array.isArray(elDef[0])) {
            var frag = document.createDocumentFragment();
            elDef.forEach(function $arr_forEach_function (def) {
                self.build(def, function $arr_build_callback (el) {
                    self.insert(el, frag, null);
                });
            });
            return callback ? callback(frag): frag;
        } else {
            if (typeof elDef === 'string') {
                return callback ?
                    callback(document.createTextNode(elDef)) :
                    document.createTextNode(elDef);
            } else {
                var el = document.createElement(elDef[0]),
                    startOn = 1;
                if (!Array.isArray(elDef[1]) && typeof elDef[1] === 'object') {
                    Object.keys(elDef[1]).forEach(function $forEach_function (key) {
                        el.setAttribute(key, elDef[1][key]);
                    });
                    startOn += 1;
                }
                (function $build (i) {
                    if (elDef[i]) {
                        return self.build(elDef[i], function $build_callback (subEl) {
                            return self.insert(subEl, el, function $insert_callback () {
                                return setTimeout(function () {
                                    $build(i + 1);
                                }, 0);
                            });
                        });
                    } else {
                        return;
                    }
                }(startOn));
                return callback ? callback(el): el;
            }
        }
    };
    /**
     * Appends a DocumentFragment (or related object) to the
     * target elements
     *
     * Fragment -- insert into --> target
     */
    self.insert = function (frag, target, callback) {
        if (typeof target === 'string' && target.charAt(0) === '#') {
            target = document.getElementById(target.match(/^#(.*)$/)[1]);
        }
        target.appendChild(frag);
        return callback ? callback(target) : target;
    };
    /**
     * Deletes an element or an array of elements and returns a
     * a reference or an array of references to the parent(s) of
     * the deleted element(s).
     */
    self.destroy = function (target, callback) {
        var parent = [],
            targets = Array.isArray(target) ||
                target instanceof NodeList ? target : [target];
        targets.forEach(function (t) {
            parent.push(t.parentNode);
            t.parentNode.removeChild(t);
        });
        parent = parent.length === 1 ? parent[0] : parent;
        return callback ? callback(parent) : parent;
    };
};
